# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 07.10.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 7, task 2.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation

import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns # pylint: disable=import-error,unused-import

def compute_normal(data1_x, data1_y, data2_x, data2_y, y_min, y_max):
  """
  Computes the line which points are equidistant to the mean values of data1
  and data2.

  A line (x, y) through a point (x0, y0) and normal to the line given by the
  vector n=[a, b] will obey the following

  a(x - x0) + b(y - y0) = 0.

  In this case, the vector between the mean points is our normal vector, and
  the point it should pass through is the average between the means.
  """

  mean1 = (np.mean(data1_x), np.mean(data1_y))
  mean2 = (np.mean(data2_x), np.mean(data2_y))
  avg12 = ((mean1[0] + mean2[0])/2, (mean1[1] + mean2[1])/2)

  # Compute min and max for x (mostly for plotting reasons)
  x_min = avg12[0] - (mean2[1] - mean1[1])/(mean2[0] - mean1[0])*(y_min - avg12[1])
  x_max = avg12[0] - (mean2[1] - mean1[1])/(mean2[0] - mean1[0])*(y_max - avg12[1])

  # Compute the line equidistant to both means
  x = np.linspace(x_min, x_max)
  y = avg12[1] - (mean2[0] - mean1[0])/(mean2[1] - mean1[1])*(x - avg12[0])

  return x, y

def main():
  """main"""

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 7')
  print('Task 2')
  print('-'*80)

  # Read Fisher's Iris data (https://en.wikipedia.org/wiki/Iris_flower_data_set)
  # from the provided comma separated list
  irisdata_file = 'data/fisheriris.txt'

  with open(irisdata_file) as fil:
    lines = fil.readlines()

  setosa_petal_width = []
  setosa_petal_length = []
  versicolor_petal_width = []
  versicolor_petal_length = []
  virginica_petal_width = []
  virginica_petal_length = []
  for line in lines:
    [_, _, petal_length, petal_width, name] = line.split(',')
    name = name.rstrip() # Remove newline character '\n'
    if name == 'Iris-setosa':
      setosa_petal_width.append(float(petal_width))
      setosa_petal_length.append(float(petal_length))
    elif name == 'Iris-versicolor':
      versicolor_petal_width.append(float(petal_width))
      versicolor_petal_length.append(float(petal_length))
    elif name == 'Iris-virginica':
      virginica_petal_width.append(float(petal_width))
      virginica_petal_length.append(float(petal_length))


  # Show dataset
  plt.figure(0)
  xmin = 0
  xmax = 7
  ymin = 0
  ymax = 4
  plt.scatter(setosa_petal_length, setosa_petal_width, c='red',
              marker='o', label='Iris setosa')
  plt.scatter(versicolor_petal_length, versicolor_petal_width, c='yellow',
              marker='o', label='Iris versicolor')
  plt.scatter(virginica_petal_length, virginica_petal_width, c='blue',
              marker='o', label='Iris virginica')
  plt.legend(loc='upper left')
  plt.xlim(xmin, xmax)
  plt.ylim(ymin, ymax)
  plt.xlabel('Petal length [cm]')
  plt.ylabel('Petal width [cm]')
  plt.axes().set_aspect('equal')
  plt.tight_layout()

  # Compute mean and decision lines
  setosa_mean = (np.mean(setosa_petal_length), np.mean(setosa_petal_width))
  versicolor_mean = (np.mean(versicolor_petal_length), np.mean(versicolor_petal_width))
  virginica_mean = (np.mean(virginica_petal_length), np.mean(virginica_petal_width))

  # Show dataset with mean and decision lines

  # The decision line between two points (x_1, y_1) and (x_2, y_2) is every point
  # on a line equidistant to both points.

  xsve, setosa_versicolor_normal = compute_normal(setosa_petal_length,
                                                  setosa_petal_width,
                                                  versicolor_petal_length,
                                                  versicolor_petal_width,
                                                  ymin, ymax)
  xsvi, setosa_virginica_normal = compute_normal(setosa_petal_length,
                                                 setosa_petal_width,
                                                 virginica_petal_length,
                                                 virginica_petal_width,
                                                 ymin, ymax)
  xvv, virginica_versicolor_normal = compute_normal(virginica_petal_length,
                                                    virginica_petal_width,
                                                    versicolor_petal_length,
                                                    versicolor_petal_width,
                                                    ymin, ymax)



  plt.figure(1)
  # Data
  plt.scatter(setosa_petal_length, setosa_petal_width, c='red',
              marker='o', label='Iris setosa')
  plt.scatter(versicolor_petal_length, versicolor_petal_width, c='yellow',
              marker='o', label='Iris versicolor')
  plt.scatter(virginica_petal_length, virginica_petal_width, c='blue',
              marker='o', label='Iris virginica')

  # Mean values
  plt.scatter(setosa_mean[0], setosa_mean[1], c='red', marker='d',
              s=100, label='Setosa mean')
  plt.scatter(versicolor_mean[0], versicolor_mean[1], c='yellow', marker='d',
              s=100, label='Versicolor mean')
  plt.scatter(virginica_mean[0], virginica_mean[1], c='blue', marker='d',
              s=100, label='Virginica mean')

  # decision boundaries
  plt.plot(xsve, setosa_versicolor_normal, label='Setosa - versicolor', c='orange')
  plt.plot(xsvi, setosa_virginica_normal, label='Setosa - virginica', c='magenta')
  plt.plot(xvv, virginica_versicolor_normal, label='Virginica - versicolor', c='green')

  plt.legend(loc='upper left')
  plt.xlim(xmin, xmax)
  plt.ylim(ymin, ymax)
  plt.xlabel('Petal length [cm]')
  plt.ylabel('Petal width [cm]')
  plt.axes().set_aspect('equal')
  plt.tight_layout()

  plt.show()

if __name__ == "__main__":
  main()
